function [MI,gMI]=fg_MI_approx(W,params)
%%---------------------------------------------------------
% function [MI,gMI]=fg_MI_approx(W,params)
% calculates the Mutual information (MI) and its gradient (gMI) as a function
% of the separation matrix (W) for use in optimization procedures.
% The entropy is approximated with no parametric kernel estimator.
% For details see: 
% S. Shwartz, M. Zibulevsky and Y. Y. Schechner, "ICA using
% Kernel Entropy Estimation with NlogN Complexity", Proc.
% of ICA2004.
% W - should be reorderd as colomn vector, W=W(:).
% params - is a structure containing the following parameters
% params.alpha = STD of Kernel window
% params.s = mixed signals, ordered as row vector in a (nsrc X T) matrix.
%   it is recomended to subtract the mean from the signals, and divide then by their STD.
% params.lambda = penalty for source scale.
% params.bins = number of gris bins.
% Bug reports:
% psarit@tx.technion.ac.il; mzib@ee.technion.ac.il; yoav@ee.technion.ac.il
%%-----------------------------------------------------------

% some global variabled for printing during optimization
global NGRAD nnormg A  qqual Lval;
alpha=params.alpha; 
s=params.s;
lambda=params.lambda;
bins=params.bins;
[S_num,Nx]=size(s); 
W=reshape(W,S_num,S_num);
%signals number and sample size
%grad of joint entropy
if abs(det(W))<1e-12
    grad_jointH=0;
    joint_H=log(1e-12);
else
    grad_jointH=inv(W)';
    joint_H=log(abs(det(W)));
end
%grad of single entropy parzen method
y=W*s;
dy=zeros(S_num,Nx);
Hy=0;
for k=1:S_num
    [Hyk, dHk]=one_entropy_gappr(y(k,:),bins,alpha,lambda);
    Hy=Hy+Hyk;
    dy(k,:)=dHk';
end
dw=dy*s';;
MI=Hy-joint_H;
gMI=dw-grad_jointH;
gMI=gMI(:);
if nargout==2
  nnormg=[nnormg norm(gMI)];
  qqual=[qqual max(sep_quality(W*A))];
  Lval=[Lval MI];
  if mod(NGRAD,1)==0, 	figure(99);
	subplot(311); semilogy(nnormg); title('Gradient norm'); 
    subplot(312); semilogy(qqual); title('Interf/signal ratio'); 
    subplot(313); plot(Lval); title('MI value'); drawnow;
  end
  NGRAD=NGRAD+1;
end